( cd usr/openwin/bin ; rm -rf shelltool )
( cd usr/openwin/bin ; ln -sf  cmdtool shelltool )
( cd usr/openwin/lib ; rm -rf text_extras_menu )
( cd usr/openwin/lib ; ln -sf  .text_extras_menu text_extras_menu )
( cd usr/openwin/lib ; rm -rf textswrc )
( cd usr/openwin/lib ; ln -sf  .textswrc textswrc )
( cd usr/openwin/lib ; rm -rf ttyswrc )
( cd usr/openwin/lib ; ln -sf  .ttyswrc ttyswrc )
( cd usr/openwin/lib ; rm -rf Xinitrc )
( cd usr/openwin/lib ; ln -sf  /var/openwin/lib/Xinitrc Xinitrc )
