/*
 * simple_panel.c -- program to display a panel button that says "Quit".
 * Selecting the panel button exits the program.
 */
#include <xview/xview.h>
#include <xview/frame.h>
#include <xview/panel.h>

main (argc, argv)
int argc;
char *argv[];
{
    Frame  frame;
    Panel  panel;
    void   quit();

    xv_init (XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

    frame = (Frame)xv_create(NULL, FRAME,
        FRAME_LABEL,    argv[0],
        XV_WIDTH,       200,
        XV_HEIGHT,      100,
        NULL);

    panel = (Panel)xv_create(frame, PANEL, NULL);

    (void) xv_create(panel, PANEL_BUTTON,
        PANEL_LABEL_STRING,  "Quit",
        PANEL_NOTIFY_PROC,   quit,
        NULL);

    xv_main_loop(frame);
}

void
quit()
{
    exit(0);
}
